#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of svgis.
# https://github.com/fitnr/svgis

# Licensed under the GNU General Public License v3 (GPLv3) license:
# http://opensource.org/licenses/GPL-3.0
# Copyright (c) 2016, Neil Freeman <contact@fakeisthenewreal.org>


class SvgisError(Exception):
    def __init__(self, arg):
        super(SvgisError, self).__init__(arg)



# -*- coding: utf-8 -*-
# This file is part of svgis.
# https://github.com/fitnr/svgis

# Licensed under the GNU General Public License v3 (GPLv3) license:
# http://opensource.org/licenses/GPL-3.0
# Copyright (c) 2015-16, Neil Freeman <contact@fakeisthenewreal.org>

from __future__ import division
import os.path
from collections import Iterable
from functools import partial
import logging
import fiona
import fiona.transform
import fionautil.scale
from . import convert, clip, draw, errors, projection, svg
from . import style as _style
from .utils import isinf

"""
Draw geodata layers into SVGs.
"""


STYLE = ('polyline, line, rect, path, polygon, .polygon {'
         ' fill: none;'
         ' stroke: #000;'
         ' stroke-width: 1px;'
         ' stroke-linejoin: round;'
         '}')


def map(layers, bounds=None, scale=None, padding=0, **kwargs):
    '''
    Draw a geodata layer to SVG. This is shorthand for creating a :class:`SVGIS` instance
    and immediately runnning :class:`SVGIS.compose`.

    Args:
        layers (sequence): Input geodata files.
        output (path): Output file name
        bounds (sequence): (minx, miny, maxx, maxy)
        scale (int): Map scale. Larger numbers -> smaller maps
        padding (int): Pad around bounds by this much. In projection units.
        crs (string): EPSG code, PROJ.4 string, or file containing a PROJ.4 string
        clip (bool): If true, clip features output to bounds.
        style (Sequence): Path to a css file or a css string.
        class_fields (Sequence): A comma-separated string or list of class names to
                                 use the SVG drawing.
        id_field (string): Field to use to determine id of each element in the drawing.
        inline (bool): If True, try to move CSS declarations into each element.

    Returns:
        String (unicode in Python 2) containing an entire SVG document.
    '''
    scale = (1 / scale) if scale else 1
    bounds = bounds if bounds and len(bounds) == 4 else None

    # Try to read style file(s)
    styles = u''.join(_style.pick(s) for s in kwargs.pop('style', []))

    proj_method, out_crs = projection.pick(kwargs.pop('crs', None))

    class_fields = set(a for c in kwargs.pop('class_fields', []) for a in c.split(','))

    drawing = SVGIS(
        layers,
        bounds=bounds,
        scalar=scale,
        proj_method=proj_method,
        out_crs=out_crs,
        padding=padding,
        style=styles,
        clip=kwargs.pop('clip', True),
        id_field=kwargs.pop('id_field', None),
        class_fields=class_fields
    ).compose(**kwargs)

    return drawing


def _property(prop, properties):
    '''
    Map a prop name to either prop_value or prop.

    Args:
        prop (unicode): Unicode property name
        properties (dict): Contains only unicode
    '''
    if prop in properties:
        try:
            return u'' + prop + u'_' + properties[prop]
        except TypeError:
            return unicode(prop) + u'_' + unicode(properties[prop])

    return prop


def _construct_classes(classes, properties):
    if isinstance(classes, basestring):
        classes = [classes]

    props = {unicode(k): unicode(v) for k, v in properties.items()}

    classes = [_style.sanitize(_property(unicode(x), props)) for x in classes]
    return (u' '.join(classes)).strip()


def _get_classes(classlist, properties, name=None):
    '''Create a list of classes that also appear in the properties'''
    classes = [c for c in classlist if c in properties]
    if name:
        classes.insert(0, name)
    return classes


class SVGIS(object):

    """
    Draw geodata files to SVG.

    Args:
        files (list): A list of files to draw.
        bounds (Sequence): An iterable with four float coordinates in (minx, miny, maxx, maxy) format
        out_crs (dict): A proj-4 like mapping. Overrides proj_method.
        proj_method (string): A keyword for picking a projection (file, utm or local).
        style (string): CSS to add to output file
        scalar (int): Map scale to use (output coordinate are divided by this)
        padding (number): Buffer each edge by this many map units

    """

    # The bounding box in input coordinates.
    _unprojected_bounds = None

    # The crs of these bounds
    _unprojected_bounds_crs = None

    # The bounding box in output coordinates, to be determined as we draw.
    _projected_bounds = (None, None, None, None)

    in_crs = None

    def __init__(self, files, bounds=None, out_crs=None, **kwargs):
        if isinstance(files, basestring):
            self.files = [files]
        elif isinstance(files, Iterable):
            self.files = files
        else:
            raise ValueError("'files' must be a file name or list of file names")

        if bounds and len(bounds) == 4:
            self._unprojected_bounds = bounds

        self.out_crs = out_crs

        self.proj_method = kwargs.pop('proj_method', None)

        self.scalar = kwargs.pop('scalar', 1) or 1

        self.style = STYLE + (kwargs.pop('style', '') or '')

        self.padding = kwargs.pop('padding', 0) or 0

        self.clip = kwargs.pop('clip', True)

        self.log = logging.getLogger('svgis')
        self.log.info('starting SVGIS with %s', self.files)

        self.simplifier = convert.simplifier(kwargs.pop('simplify', None))

        self.id_field = kwargs.pop('id_field', None)

        self.class_fields = kwargs.pop('class_fields', [])

    def __repr__(self):
        return ('SVGIS(files={0.files}, out_crs={0.out_crs})').format(self)

    def _get_clipper(self, layer_bounds, out_bounds, scalar=None):
        '''
        Get a clipping function for the given input crs and bounds.

        Args:
            layer_bounds (tuple): The bounds of the layer.
            out_bounds (tuple): The desired output bounds (in layer coordinates).
            scalar (float): Map scale.

        Returns:
            None if in_bounds == out_bounds or clipping is off.
        '''
        scalar = scalar or self.scalar
        if self.clip and not convert.bbox_covers(out_bounds, layer_bounds):
            return clip.prepare([c * scalar for c in convert.extend_bbox(self._projected_bounds)])

        else:
            return None

    def _reprojector(self, in_crs):
        '''Return a reprojection transform from in_crs to self.out_crs.'''
        if self.out_crs != in_crs:
            return partial(fiona.transform.transform_geom, in_crs, self.out_crs)

        else:
            return None

    def set_crs(self, layer_crs, bounds):
        '''Set the output CRS, if not yet set. Also update internal mbr.'''
        if not self.in_crs:
            self.in_crs = layer_crs

        if not self.out_crs:
            # Determine projection transformation:
            # either use something passed in, a non latlong layer projection,
            # the local UTM, or customize local TM
            self.out_crs = projection.choosecrs(layer_crs, bounds, proj_method=self.proj_method)

    def _project_bounds(self, in_crs, passed_bounds):
        '''Set projected bounds, but only once.'''
        if passed_bounds and not any(self._projected_bounds):
            self._projected_bounds = projection.transform_bounds(in_crs, self.out_crs, passed_bounds)
            self._unprojected_bounds_crs = in_crs

    def _extend_bounds(self, in_crs, layer_bounds):
        '''Extend projected bounds, used when calculating bounds as we go.'''
        projected = projection.transform_bounds(in_crs, self.out_crs, layer_bounds)
        self._projected_bounds = convert.updatebounds(self._projected_bounds, projected)

    def _get_local_projected_bounds(self, layer_crs):
        '''Get the projected bounds in local terms.'''
        if layer_crs != self._unprojected_bounds_crs:
            return projection.transform_bounds(self.out_crs, layer_crs, self._projected_bounds)

    def _compose_file(self, filename, scalar, unprojected_bounds=None, **kwargs):
        '''
        Draw fiona file to an SVG group.

        Args:
            filename (string): path to a fiona-readable file
            scalar (int): map scale
            unprojected_bounds (tuple): (minx, maxx, miny, maxy) in the layer's coordinate system. 'None' values are OK.
                                        "Unprojected" here refers to the fact that we haven't transformed these bounds yet.
                                        They may well, in fact, be in a projection.
            simplifier (function): Simplification function. Defaults to self.simplify.
            class_fields (sequence): Fields to turn in the element classes (default: self.class_fields).
            id_field (string): Field to use as element ID (default: self.id_field).

        Returns:
            unicode
        '''
        self.log.info('starting %s', filename)
        with fiona.open(filename, "r") as layer:

            bounds = unprojected_bounds or self._unprojected_bounds or layer.bounds

            # Set the output CRS, if not yet set.
            self.set_crs(layer.crs, bounds)

            if unprojected_bounds or self._unprojected_bounds:
                # Use the passed bounds, if they exist.
                # This sets self._projected_bounds, and prevents it from being altered
                self._project_bounds(layer.crs, unprojected_bounds)
                # Get the bounds to use here from the projected bounds.
                # If None, the unprojected bounds are our best bet.
                bounds = self._get_local_projected_bounds(layer.crs) or unprojected_bounds or self._unprojected_bounds

            else:
                # Using bounds from all the input layers.
                # self._projected_bounds will continue to be updated.
                self._extend_bounds(layer.crs, bounds)

            transforms = [
                self._reprojector(layer.crs),
                partial(fionautil.scale.geometry, factor=scalar),
                # Get clipping function based on a slightly extended version of _projected_bounds.
                self._get_clipper(layer.bounds, bounds, scalar=scalar),
                kwargs.pop('simplifier', None)
            ]

            # Correct for OGR's lack of creativity for GeoJSONs.
            if layer.name == 'OGRGeoJSON':
                kwargs['_file_name'] = os.path.splitext(os.path.basename(filename))[0]
            else:
                kwargs['_file_name'] = layer.name

            # A list of class names to get from layer properties.
            cf = kwargs.pop('class_fields', self.class_fields)
            classes = _get_classes(cf, layer.schema['properties'], kwargs['_file_name'])

            # Remove the id field if it doesn't appear in the properties.
            id_field = kwargs.pop('id_field', self.id_field)
            layer_classes = list(layer.schema['properties'].keys())
            kwargs['id_field'] = id_field if id_field in layer_classes else None

            group = [self._feature(f, transforms, classes, **kwargs) for _, f in layer.items(bbox=bounds)]

        gargs = {
            'id': kwargs['_file_name'],
            'class': ' '.join(_style.sanitize(c) for c in layer_classes)
        }
        self.log.info('finishing %s', filename)
        return svg.group(group, **gargs)

    def _feature(self, feature, transforms, classes, id_field, **kwargs):
        '''
        Draw a single feature.

        Args:
            feature (dict): A GeoJSON like feature dict produced by Fiona.
            transforms (list): Functions to apply to the geometry.
            classes (list): Names of fields to apply as classes in the output element.
            id_field (string): Field to use as id of the output element.
            kwargs: Additional properties to apply to the element.

        Returns:
            unicode
        '''
        # Produce the geometry.
        file_name = kwargs.pop('_file_name', '?')

        try:
            geom = feature.get('geometry')

            for t in [x for x in transforms if x is not None]:
                geom = t(geom)

        except ValueError as e:
            self.log.warning("Error drawing feature %s of %s: %s", file_name, feature.get('id', '?'), e)
            return u''

        # Set up the element's properties.
        kwargs['class'] = _construct_classes(classes, feature['properties'])

        if id_field:
            kwargs['id'] = _style.sanitize(feature['properties'].get(id_field))

        try:
            return draw.geometry(geom, **kwargs)

        except errors.SvgisError as e:
            self.log.warning("Error drawing %s: %s", file_name, e)
            return u''

    def compose(self, scalar=None, bounds=None, **kwargs):
        '''
        Draw files to svg.

        Args:
            scalar (int): factor by which to scale the data, generally a small number (1/map scale).
            bounds (Sequence): Map bounding box in input units. Defaults to map data bounds.
            style (str): CSS to append to parent object CSS.
            viewbox (bool): If True, draw SVG with a viewbox. If False, translate coordinates to the frame.
                            Defaults to True.
            precision (float): Round coordinates to this precision [default: 0].
            simplify (float): Must be between 0 and 1. Fraction of removable coordinates to keep.
            inline (bool): If True, try to run CSS into each element.

        Returns:
            String (unicode in Python 2) containing an entire SVG document.
        '''
        # Set up arguments
        scalar = scalar or self.scalar

        drgs = {
            'style': kwargs.pop('style', ''),
            'viewbox': kwargs.pop('viewbox', True),
            'inline': kwargs.pop('inline', False),
        }

        if bounds:
            reset_bounds = True
        else:
            reset_bounds = False
            bounds = self._unprojected_bounds

        if 'simplify' in kwargs:
            kwargs['simplifier'] = convert.simplifier(kwargs.pop('simplify'))
        else:
            kwargs['simplifier'] = self.simplifier

        kwargs['precision'] = kwargs.get('precision', 0)

        # Draw files
        with fiona.drivers():
            members = [self._compose_file(f, scalar, unprojected_bounds=bounds, **kwargs) for f in self.files]

        drawing = self._draw(members, bounds, scalar, **drgs)

        # Reset bounds so that self can be used again fresh. This is hacky.
        if reset_bounds:
            self._projected_bounds = (None, None, None, None)

        return drawing

    def _draw(self, members, bounds, scalar, **kwargs):
        '''
        Combine drawn layers into an SVG drawing.

        Args:
            members (list): unicode representations of SVG groups.
            bounds (Sequence): Map bounding box in input units. If None, uses map data bounds.
            scalar (int): factor by which to scale the data, generally a small number (1/map scale).
            style (str): CSS to append to parent object CSS.
            viewbox (bool): If True, draw SVG with a viewbox. If False, translate coordinates to
                            the frame. Defaults to True.
            inline (bool): If True, try to run CSS into each element.

        Returns:
            String (unicode in Python 2) containing an entire SVG document.
        '''
        groupargs = {
            'transform': 'scale(1, -1) translate({},{})'.format(self.padding, -self.padding),
            'fill_rule': 'evenodd'
        }

        svgargs = {
            'style': self.style + (kwargs.pop('style', '') or '')
        }

        x0, y0, x1, y1 = self._corners(scalar, unprojected_bounds=bounds)

        # width and height
        size = [
            x1 - x0 + (self.padding * 2),
            y1 - y0 + (self.padding * 2)
        ]

        if kwargs.pop('viewbox', True):
            svgargs['viewbox'] = [x0, -y1] + size
        else:
            groupargs['transform'] += ' translate({}, {})'.format(-x0, -y1)

        # Create container and then SVG
        container = svg.group(members, **groupargs)
        drawing = svg.drawing(size, [container], **svgargs)

        if kwargs.pop('inline', False):
            self.log.info('Inlining styles')
            drawing = _style.inline(drawing, svgargs['style'])

        return drawing

    def _corners(self, scalar, unprojected_bounds=None):
        '''
        Calculate the bounds in svg coordinates.
        By default, uses self's minimum bounding rectangle.

        Args:
            scalar (int): Map scale
            bounds (Sequence): Optional bounds to calculate with (in output coordinates)

        Returns:
            tuple representing minx, miny, maxx, maxy in output coordinates
        '''
        if unprojected_bounds is None:
            bounds = self._projected_bounds
        else:
            bounds = projection.transform_bounds(self.in_crs, self.out_crs, unprojected_bounds)

        if any([isinf(b) for b in bounds]):
            self.log.warning('Drawing has infinite bounds, consider changing projection or bounding box.')

        try:
            return [i * float(scalar) for i in bounds]

        except ValueError:
            raise ValueError('Problem calculating drawing dimensions. Are bounds are in minx, miny, maxx, maxy order?')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# This file is part of svgis.
# https://github.com/fitnr/svgis

# Licensed under the GNU General Public License v3 (GPLv3) license:
# http://opensource.org/licenses/GPL-3.0
# Copyright (c) 2015-16, Neil Freeman <contact@fakeisthenewreal.org>

from types import GeneratorType
try:
    from shapely.geometry import shape, mapping
    import numpy as np
except ImportError:
    pass


def _expand_np(coordinates):
    return np.array(_expand_py(coordinates))


def _expand_py(coordinates):
    return list(coordinates)


def expand(ring):
    if isinstance(ring, GeneratorType):
        try:
            return _expand_np(ring)
        except NameError:
            return _expand_py(ring)
    else:
        return ring


def expand_rings(rings):
    try:
        return [_expand_np(ring) for ring in rings]
    except NameError:
        return [_expand_py(ring) for ring in rings]


def expand_geom(geom):
    '''Expand generators in a geometry's coordinates.'''

    if geom['type'] == 'MultiPolygon':
        geom['coordinates'] = [expand_rings(rings) for rings in geom['coordinates']]

    elif geom['type'] in ('Polygon', 'MultiLineString'):
        geom['coordinates'] = expand_rings(geom['coordinates'])

    elif geom['type'] in ('Point', 'MultiPoint', 'LineString'):
        geom['coordinates'] = expand(geom['coordinates'])

    elif geom['type'] == 'GeometryCollection':
        geom['geometries'] = [expand_geom(g) for g in geom['geometries']]

    else:
        raise NotImplementedError("Unsupported geometry type " + geom['type'])

    return geom


def prepare(bbox):
    minx, miny, maxx, maxy = bbox
    bounds = {
        "type": "Polygon",
        "coordinates": [[(minx, miny), (minx, maxy), (maxx, maxy), (maxx, miny), (minx, miny)]]
    }
    try:
        bbox = shape(bounds)

        def func(geometry):
            # This is technically only needed in Py3, but whatever.
            geom = expand_geom(geometry)
            clipped = bbox.intersection(shape(geom))
            return mapping(clipped)

    except NameError:
        def func(geometry):
            return geometry

    return func


def clip(geometry, bbox):
    '''Clip a geometry to a bounding box. BBOX may be a tuple or a shapely geometry object'''
    try:
        return prepare(bbox)(geometry)

    except NameError:
        return geometry



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of svgis.
# https://github.com/fitnr/svgis

# Licensed under the GNU General Public License v3 (GPLv3) license:
# http://opensource.org/licenses/GPL-3.0
# Copyright (c) 2015, Neil Freeman <contact@fakeisthenewreal.org>

import unittest
import sys
import os
import io
import functools
from xml.dom import minidom

import click.testing
import svgis.cli


PROJECTION = '+proj=lcc +lat_1=20 +lat_2=60 +lat_0=40 +lon_0=-96 +x_0=0 +y_0=0 +datum=NAD83 +units=m +no_defs'
BOUNDS = (-124.0, 20.5, -64.0, 49.0)


class CliTestCase(unittest.TestCase):
    fixture = 'tests/test_data/test.svg'

    shp = 'tests/test_data/cb_2014_us_nation_20m.shp'
    css = 'polygon{fill:green}'

    def setUp(self):
        self.runner = runner = click.testing.CliRunner()
        self.invoke = functools.partial(runner.invoke, svgis.cli.main, catch_exceptions=False)

    def testSvgStyle(self):
        sys.stdout = io.StringIO()
        self.invoke(['style', '-s', self.css, self.fixture, 'tmp.svg'])
        try:
            with open('tmp.svg') as f:
                self.assertIn(self.css, f.read()[0:1000])
        finally:
            os.remove('tmp.svg')

    def testSvgScale(self):
        self.invoke(['scale', '-f', '123', self.fixture, 'tmp.svg'])
        try:
            with open('tmp.svg') as f:
                self.assertIn('scale(123)', f.read()[0:1000])

        finally:
            os.remove('tmp.svg')

    def testSvgProjectUtm(self):
        p = self.invoke(['project', '-m', 'utm', '--', '-110.277906', '35.450777', '-110.000477', '35.649030'])
        expected = '+proj=utm +zone=12 +north +datum=WGS84 +units=m +no_defs\n'
        self.assertEqual(p.output, expected)

    def testSvgProject(self):
        p = self.invoke(['project', '--', '-110.277906', '35.450777', '-110.000477', '35.649030'])

        try:
            self.assertEqual(p.exit_code, 0)
        except AssertionError:
            # print(p.exc_info)
            raise

        expected = ('+proj=lcc +lon_0=-111.0 +lat_1=35.64903 +lat_2=35.450777 '
                    '+lat_0=35.64903+x_0=0 +y_0=0 +ellps=GRS80 '
                    '+towgs84=0,0,0,0,0,0,0+units=m +no_defs\n')

        self.assertEqual(p.output, expected)

    def testCliDraw(self):
        self.invoke(['draw', '-j', PROJECTION, '-f', '1000', self.shp,
                     '-o', 'tmp.svg', '--bounds'] + [str(b) for b in BOUNDS])

        try:
            result = minidom.parse('tmp.svg').getElementsByTagName('svg').item(0)
            fixture = minidom.parse(self.fixture).getElementsByTagName('svg').item(0)

            result_vb = [float(x) for x in result.attributes.get('viewBox').value.split(',')]
            fixture_vb = [float(x) for x in fixture.attributes.get('viewBox').value.split(',')]

            for r, f in zip(result_vb, fixture_vb):
                self.assertAlmostEqual(r, f, 5)

        finally:
            os.remove('tmp.svg')

    def testCliHelp(self):
        result = self.invoke(('--help',))
        self.assertEqual(result.exit_code, 0)

        result = self.invoke(['style', '--help'])
        assert result.exit_code == 0

        result = self.invoke(['draw', '--help'])
        assert result.exit_code == 0

        result = self.invoke(['project', '--help'])
        assert result.exit_code == 0

        result = self.invoke(['scale', '--help'])
        assert result.exit_code == 0

    def testErrs(self):
        with self.assertRaises(IOError):
            self.invoke(('draw', 'lksdjlksjdf'))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of svgis.
# https://github.com/fitnr/svgis

# Licensed under the GNU General Public License v3 (GPLv3) license:
# http://opensource.org/licenses/GPL-3.0
# Copyright (c) 2015, Neil Freeman <contact@fakeisthenewreal.org>

from __future__ import unicode_literals
import unittest
import functools
import collections
from svgis import convert


class ConvertTestCase(unittest.TestCase):

    def test_rect(self):
        pass

    def test_updatebounds(self):
        bounds1 = (None, 0.1, None, 1.1)
        bounds2 = (0.2, 0.2, 1.2, 1.2)
        bounds3 = (0.1, 0.3, 1.5, 1.1)
        bounds4 = (0.05, 0.4, float('inf'), 1.2)
        bounds5 = (0.05, -1 * float('inf'), 1.4, 1.2)

        self.assertSequenceEqual(convert.updatebounds(bounds1, bounds2), (0.2, 0.1, 1.2, 1.2))
        self.assertSequenceEqual(convert.updatebounds(bounds3, bounds2), (0.1, 0.2, 1.5, 1.2))
        self.assertSequenceEqual(convert.updatebounds(bounds3, bounds4), (0.05, 0.3, 1.5, 1.2))
        self.assertSequenceEqual(convert.updatebounds(bounds3, bounds5), (0.05, 0.3, 1.5, 1.2))

    def testConvertBbox(self):
        bounds = (-100, -100, 100, 100)

        assert convert.extend_bbox(bounds, ext=100) == (-200, -200, 200, 200)
        assert convert.extend_bbox(bounds, ext=10) == (-110, -110, 110, 110)

    def testSimplify(self):
        a = convert.simplifier(None)
        assert a.__name__ == '<lambda>'
        assert isinstance(a, collections.Callable)

        b = convert.simplifier(100)
        assert b.__name__ == '<lambda>'
        assert isinstance(b, collections.Callable)

        c = convert.simplifier(50)

        try:
            import visvalingamwyatt
            assert isinstance(c, functools.partial)
        except ImportError:
            assert c.__name__ == '<lambda>'

    def test_bbox_covers(self):
        b1 = (0, 0, 10, 10)
        b2 = (0, 0, 20, 10)
        b3 = (0, 0, 5, 11)

        assert convert.bbox_covers(b1, b2) is False
        assert convert.bbox_covers(b2, b1) is True
        assert convert.bbox_covers(b1, b3) is False
        assert convert.bbox_covers(b3, b1) is False
        assert convert.bbox_covers(b3, b3) is True

    def testbounds_to_ring(self):
        fix = [
            (0, 0), (0, 0.6), (0, 1.2), (0, 1.7999999999999998), (0, 2.4), (0, 3.0),
            (0, 3.6), (0, 4.2), (0, 4.8), (0, 5.3999999999999995), (0, 5.999999999999999),
            (0.6, 6), (1.2, 6), (1.7999999999999998, 6), (2.4, 6), (3.0, 6), (3.6, 6), (4.2, 6),
            (4.8, 6), (5.3999999999999995, 6), (5.999999999999999, 6), (6, 5.999999999999999),
            (6, 5.3999999999999995), (6, 4.8), (6, 4.2), (6, 3.6), (6, 3.0), (6, 2.4), (6, 1.7999999999999998),
            (6, 1.2), (6, 0.6), (6, 0), (5.999999999999999, 0), (5.3999999999999995, 0), (4.8, 0), (4.2, 0),
            (3.6, 0), (3.0, 0), (2.4, 0), (1.7999999999999998, 0), (1.2, 0), (0.6, 0), (0, 0)
        ]

        r = convert.bounds_to_ring(0, 0, 6, 6)
        self.assertSequenceEqual(r, fix)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of svgis.
# https://github.com/fitnr/svgis

# Licensed under the GNU General Public License v3 (GPLv3) license:
# http://opensource.org/licenses/GPL-3.0
# Copyright (c) 2015, Neil Freeman <contact@fakeisthenewreal.org>

from setuptools import setup

try:
    readme = open('README.rst').read()
except IOError:
    readme = ''

setup(
    name='svgis',

    version='0.3.6',

    description='Draw geodata in SVG',

    long_description=readme,

    keywords='svg gis geojson shapefile',

    author='Neil Freeman',

    author_email='contact@fakeisthenewreal.org',

    url='https://github.com/fitnr/svgis',

    license='GNU General Public License v3 (GPLv3)',

    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Operating System :: Unix',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Operating System :: OS Independent',
    ],

    packages=['svgis'],

    package_data={
        'svgis': [
            'test_data/test.*',
            'test_data/*.json'
        ]
    },

    install_requires=[
        'click>=6.2,<6.3',
        'pyproj>=1.9.5,<1.10',
        'fiona>=1.6.0,<2.0',
        'fionautil >=0.4.11, <1.0',
        'tinycss>=0.3,<0.4',
        'utm>=0.4.0,<1'
    ],

    extras_require={
        'numpy': [''],
        'clip': ['shapely>=1.5.7'],
        'inline': [],
        'simplify': ['visvalingamwyatt>=0.1.1']
    },

    test_suite='tests',

    tests_require=['six'],

    entry_points={
        'console_scripts': [
            'svgis=svgis.cli:main',
        ],
    },

    use_2to3=True,
)
